#!/usr/bin/env python
# encoding: utf-8
"""
$Id: adl_atms_sdr.py 2340 2015-01-22 13:40:38Z scottm $

Purpose: Run the ATMS SDR using Raytheon ADL 3.

Input:
    One or more HDF5 ATMS RDR input files, aggregated or single-granule.
    You need one granule before and one granule after a given granule to process.
    A work directory, typically, empty, in which to unpack the granules and generate the output.
    If the work directory specified does not exist, it will be created.

Output:
    ADL ATMS SDR blob files, .asc metadata files, and HDF5 output granules will be created.

Details:
    If you input a series of granules, the software will scan the work directory.
    Thus, for N input RDR granules, you may get up to N-2 output SDR granules if they are all contiguous.
    It is ambiguous to provide several copies of the same granule in the work directory;
    this will result in an error abort.
    The unpacker gives each unpacking of a given granule its own

Preconditions:
    Requires ADL_HOME, CSPP_RT_ANC_PATH, CSPP_RT_ANC_CACHE_DIR environment variables are set.
    Requires that any needed LD_LIBRARY_PATH is set.
    Requires that DSTATICDATA is set.

Optional:
    Environment variables CSPP_RT_ANC_PATH, CSPP_RT_ANC_CACHE_DIR, if static ancillary
     data is not placed within ADL_HOME.

Created Oct 2011 by R.K.Garcia <rayg@ssec.wisc.edu>
Copyright (c) 2011-2012 University of Wisconsin Regents.
Licensed under GNU GPLv3.
"""

import os
import sys
import logging
import glob
import traceback

import datetime as dt
from subprocess import CalledProcessError, call
from shutil import rmtree
from pprint import pprint
from multiprocessing import Pool, Lock, Value, cpu_count
import shutil
# skim and convert routines for reading .asc metadata fields of interest
from adl_asc import skim_dir, contiguous_granule_groups, granule_groups_contain, effective_anc_contains, \
    RDR_REQUIRED_KEYS, POLARWANDER_REQUIRED_KEYS

# post-processing on blob+asc products
from adl_post_process import repack_products, aggregate_products_par, add_geo_attribute_to_aggregates
from adl_post_process import SHORTNAME_2_PRODUCTID
import rdr_unpack


# ancillary search and unpacker common routines
from adl_common import sh, anc_files_needed, link_ancillary_to_work_dir, configure_logging, \
    env, h5_xdr_inventory, get_return_code, check_env
from adl_common import check_anc_cache, ADL_HOME, CSPP_RT_ANC_PATH, CSPP_RT_ANC_CACHE_DIR, COMMON_LOG_CHECK_TABLE, \
    check_and_convert_path, CSPP_RT_ANC_TILE_PATH, status_line,check_existing_env_var

# log file scanning
import adl_log
from adl_atms_data import ADL_ATMS_ANC_GLOBS,GIG_MEMORY_PER_PROC,ADL_ATMS_PRODUCT_IDS,ATMS_GRAN_ID_DELTA

# cache loading and searching
import adl_anc_retrieval

# N_GEO_Ref fix for ADL
from adl_geo_ref import write_geo_ref

from sdr_common import build_workspace,setup_multi_process, trim_the_granules, start_out_good, report_gran_completion, number_good_granules, check_memory,check_number_processors,move_products_to_work_directory

# every module should have a LOG object
LOG = logging.getLogger('adl_atms_sdr')

ADL_ATMS_SDR = os.path.join(ADL_HOME, 'bin', 'ProSdrAtmsController.exe')

# keys used in metadata dictionaries
K_FILENAME = '_asc_filename'

# directories in which we find the ancillary files, including CC-Int file for ATMS SDR
ADL_ANC_DIRS = CSPP_RT_ANC_PATH



# filenames which we'll need to set N_GEO_Ref on
# FIXME: this should be derived from product lists and not from filesystem search?
ADL_ATMS_SDR_GLOBS = ('SATMS*.h5', 'TATMS*.h5')

# we're not likely to succeed in processing using granules smaller than this many bytes
MINIMUM_RDR_BLOB_SIZE = 100000

MAX_CONTIGUOUS_DELTA_CRIS_RTSTPS_5_3 = 30    # in seconds; this accounts for defects in RT-STPS 5.3

# attribute paths for SDRs and TDRs
# FIXME: review this
SDR_GRANULE_ID_ATTR_PATH = 'Data_Products/ATMS-SDR/ATMS-SDR_Gran_0/N_Granule_ID'
TDR_GRANULE_ID_ATTR_PATH = 'Data_Products/ATMS-TDR/ATMS-TDR_Gran_0/N_Granule_ID'


# products which we aggregate
ATMS_SDR_PRODUCTS_AGG = [
    'ATMS-SDR',
    #'ATMS-FSDR',
    'ATMS-TDR',
    'ATMS-SDR-GEO'
]

# products which we aggregate and assign N_GEO_Ref to
ATMS_SDR_PRODUCTS_AGG_GEO = [
    'ATMS-SDR',
    'ATMS-TDR'
]

# intermediate products we're not really doing anything with
ATMS_SDR_PRODUCTS_NOAGG = [
    'ATMS-SDR-IP'
]


# XML template for ProSdrAtmsController.exe
XML_TMPL_ATMS_SDR = """<InfTkConfig>
  <idpProcessName>ProSdrAtmsController.exe</idpProcessName>
  <siSoftwareId></siSoftwareId>
  <isRestart>FALSE</isRestart>
  <useExtSiWriter>FALSE</useExtSiWriter>
  <debugLogLevel>NORMAL</debugLogLevel>
  <debugLevel>DBG_HIGH</debugLevel>
  <enablePerf>FALSE</enablePerf>
  <perfPath>${WORK_DIR}/log</perfPath>
  <dbgPath>${WORK_DIR}/log</dbgPath>
  <initData>
     <domain>OPS</domain>
     <subDomain>SUBDOMAIN</subDomain>
     <startMode>INF_STARTMODE_COLD</startMode>
     <executionMode>INF_EXEMODE_PRIMARY</executionMode>
     <healthTimeoutPeriod>30</healthTimeoutPeriod>
  </initData>
  <lockinMem>FALSE</lockinMem>
  <rootDir>${WORK_DIR}/log</rootDir>
  <inputPath>${WORK_DIR}:${LINKED_ANCILLARY}:${CSPP_RT_ANC_TILE_PATH}</inputPath>
  <outputPath>${GRANULE_OUTPUT_DIR}</outputPath>
  <dataStartIET>0000000000000000</dataStartIET>
  <dataEndIET>1111111111111111</dataEndIET>
  <actualScans>12</actualScans>
  <previousActualScans>12</previousActualScans>
  <nextActualScans>12</nextActualScans>
  <usingMetadata>TRUE</usingMetadata>
  <configGuideName>ProSdrAtms_GuideList.cfg</configGuideName>

  <task>
    <taskType>SDR</taskType>
    <taskDetails1>%(N_Granule_ID)s</taskDetails1>
    <taskDetails2>%(N_Granule_Version)s</taskDetails2>
    <taskDetails3>NPP</taskDetails3>
    <taskDetails4>ATMS</taskDetails4>
  </task>

  <task>
    <taskType>SHUTDOWN</taskType>
    <taskDetails1></taskDetails1>
    <taskDetails2></taskDetails2>
    <taskDetails3></taskDetails3>
    <taskDetails4></taskDetails4>
  </task>
</InfTkConfig>
"""


def _skim_atms_science_rdr(work_dir):
    "skim for ATMS-SCIENCE-RDR data meeting minimum requirements"
    for info in skim_dir(work_dir, required_keys=RDR_REQUIRED_KEYS, N_Collection_Short_Name="ATMS-SCIENCE-RDR"):
        path = info['BlobPath']
        if not os.path.isfile(path) or os.stat(path).st_size < MINIMUM_RDR_BLOB_SIZE:
            LOG.info('Ignoring %s, invalid blob. For direct broadcast this can happen at start or end of pass.' % path)
        else:
            yield info


def sift_metadata_for_atms_sdr(work_dir='.'):
    """
    search through the ASC metadata in a directory,
    grouping in StartTime order
    look for back-to-back granules and break into contiguous sequences
    check that S/C diary RDRs are available for granules of interest
    yield sequence of granules we can process
    """
    LOG.debug('collecting information for S/C diary RDRs')
    # FIXME: a bug in RT-STPS incremental RDR granule production makes a
    # short S/C Diary RDR at the start of each natural boundary, which we want to ignore for now
    # eventually the larger_granules_preferred option should go away
    diaries = list(contiguous_granule_groups(skim_dir(work_dir,
                                                      required_keys=RDR_REQUIRED_KEYS,
                                                      N_Collection_Short_Name='SPACECRAFT-DIARY-RDR'),
                                             larger_granules_preferred=True))
    if len(diaries) == 0:
        LOG.error('No spacecraft diary files were found!')

    LOG.debug('sifting science RDR data for processing opportunities')
    for group in contiguous_granule_groups(_skim_atms_science_rdr(work_dir),
                                           tolerance=dt.timedelta(seconds=MAX_CONTIGUOUS_DELTA_CRIS_RTSTPS_5_3)):
        # for ATMS, we can process everything but the first and last granule
        # for CrIS, use [4:-4]
        LOG.debug('contiguous granule group: %r' % (group,))
        for gran in group[1:-1]:
            if not granule_groups_contain(diaries, gran):
                LOG.info("Insufficient S/C Diary RDR coverage to process %s @ %s (%s) - skipping" % (
                    gran['N_Granule_ID'], gran['StartTime'], gran['URID']))
                continue
            LOG.debug('Processing opportunity: %r at %s with uuid %s' % (
                gran['N_Granule_ID'], gran['StartTime'], gran['URID']))
            yield gran


# def generate_sdr_xml(work_dir, granule_seq):
#     "generate pairs of granule name,XML file for ATMS SDR granule generation"
#     to_process = []
#     for gran in granule_seq:
#         name = gran['N_Granule_ID']
#         fnxml = 'sdr_atms_%s.xml' % name
#         LOG.debug('writing XML file %r' % fnxml)
#         fpxml = file(os.path.join(work_dir, fnxml), 'wt')
#         fpxml.write(XML_TMPL_ATMS_SDR % gran)
#         to_process.append((name, fnxml))
#     return to_process

def generate_sdr_xml(work_dir, gran):
    "generate pairs of granule name,XML file for ATMS SDR granule generation"

    name = gran['N_Granule_ID']
    fnxml = 'sdr_atms_%s.xml' % name
    LOG.debug('writing XML file %r' % fnxml)
    fpxml = file(os.path.join(work_dir, fnxml), 'wt')
    fpxml.write(XML_TMPL_ATMS_SDR % gran)

    return fnxml


## Look through new log files for completed messages
## based on CrIS SDR original
#def check_log_files(work_dir, pid, xml):
#    """
#    Find the log file
#    Look for success
#    Return True if found
#    Display log message and hint if problem occurred
#    """
#    # retrieve exe name and log path from lw file
#    logpat = os.path.join(work_dir, "log", "*%d*.log" % pid)
#
#    n_err=0
#    for logFile in glob.glob(logpat):
#        LOG.debug( "Checking Log file " +logFile +" for errors.")
#        n_err += adl_log.scan_log_file(COMMON_LOG_CHECK_TABLE, logFile)
#
#        if n_err == 0 :
#            LOG.debug(" Log: "+ logFile + " "+ xml + " Completed successfully" )
#            return True
#        else :
#            LOG.error( " Log: "+ logFile +" " + xml + " Completed unsuccessfully" )
#            return False
#        
# Look through new log files for completed messages
# based on CrIS SDR original
def check_log_files(work_dir, pid, xml):
    """
    Find the log file
    Look for success
    Return True if found
    Display log message and hint if problem occurred
    """
    # retrieve exe name and log path from lw file
    logpat = os.path.join(work_dir, "log", "*%d*.log" % pid)

    zult = True
    for logFile in glob.glob(logpat):
        LOG.debug("Checking Log file " + logFile + " for errors.")
        n_err = adl_log.scan_log_file(COMMON_LOG_CHECK_TABLE, logFile)

        if n_err == 0:
            LOG.debug(" Log: " + logFile + " " + xml + " Completed successfully")
        else:
            LOG.error(" Log: " + logFile + " " + xml + " Completed unsuccessfully")
            zult = False

    return zult




# def move_products_to_work_directory(work_dir):
#     """ Checks that proper products were produced.
#     If product h5 file was produced blob and asc files are deleted
#     If product was not produced files are left alone
#     """
#     dest = os.path.dirname(work_dir)
#
#     files = glob.glob(os.path.join(work_dir, "*.h5"))
#     for file in files:
#         shutil.move(file, dest)
#
#     return
def products_in_directory(work_dir,product_ids):
    """ Checks that proper products were produced.
    If product h5 file was produced blob and asc files are deleted
    If product was not produced files are left alone
    """


    for product_id in product_ids:
        files = glob.glob(os.path.join(work_dir, '%s*.h5'%product_id))
        for afile in files:
            yield afile
    return

def run_and_check_single_granule(additional_env):
    "run a VIIRS SDR XML input in sequence"



    good = True

    work_dir = additional_env['WORK_DIR']
    xml = additional_env['XML_FILE']
    granule_id = additional_env['N_Granule_ID']
    granule_output_dir = additional_env['GRANULE_OUTPUT_DIR']

    satpat = os.path.join(granule_output_dir, 'SATMS*.h5')
    tatpat = os.path.join(granule_output_dir, 'TATMS*.h5')

    _, tatms = h5_xdr_inventory(tatpat, TDR_GRANULE_ID_ATTR_PATH)
    prior_granules, satms = h5_xdr_inventory(satpat, SDR_GRANULE_ID_ATTR_PATH)

    cmd = [ADL_ATMS_SDR, xml]

    LOG.debug('executing "%s"' % ' '.join(cmd))
    LOG.debug('additional environment variables: %s' % repr(additional_env))

    try:
        pid = sh(cmd, env=env(**additional_env), cwd=work_dir)
        LOG.debug("%r ran as pid %d" % (cmd, pid))
    #        if not check_log_files(work_dir, pid, xml):
    #            bad_log_runs.add(granule_id)

    except CalledProcessError as oops:
        pid = getattr(oops, 'pid', None)
        #        if not check_log_files(work_dir, pid, xml):
        #            bad_log_runs.add(granule_id)

        LOG.debug(traceback.format_exc())
        LOG.error('%s failed on %r: %r. Continuing...' % (cmd[0], xml, oops))
        #        crashed_runs.add(granule_id)
        good = False

    # note: this is done differently than for aggregated granules
    new_granules, satms = h5_xdr_inventory(satpat, SDR_GRANULE_ID_ATTR_PATH, state=satms)
    LOG.debug('new SDR granules after this run: %s' % repr(new_granules))
    if granule_id not in new_granules:
        LOG.warning('no SDR HDF5 output for %s' % granule_id)
    #        no_output_runs.add(granule_id)
        good = False
    # else:
    #     filename = new_granules[granule_id]
    #     try:
    #         geo_file, definitive_match = write_geo_ref(filename)
    #         if not definitive_match:
    #             LOG.warning("GEO file %s for %s is not a definitive match" % (geo_file, filename))
    #     except ValueError as oops:
    #         LOG.warning("problem setting /N_GEO_Ref on %s" % filename)
    #         #            geo_problem_runs.add(granule_id)
    #         good = False

    for afile in products_in_directory(granule_output_dir, ("SATMS", "TATMS")):
        LOG.info( afile)
        try:
            geo_file, definitive_match = write_geo_ref(afile)
            if not definitive_match:
                LOG.warning("GEO file %s for %s is not a definitive match" % (geo_file, afile))
        except ValueError :
            LOG.warning("problem setting /N_GEO_Ref on %s" % afile)




    # # check new TATMS output granules
    # new_granules, tatms = h5_xdr_inventory(tatpat, TDR_GRANULE_ID_ATTR_PATH, state=tatms)
    # LOG.debug('new TDR granules after this run: %s' % repr(new_granules))
    # if granule_id not in new_granules:
    #     LOG.debug('no TDR HDF5 output for %s' % granule_id)
    # else:
    #     filename = new_granules[granule_id]
    #     try:
    #         geo_file, definitive_match = write_geo_ref(filename)
    #         if not definitive_match:
    #             LOG.warning("GEO file %s for %s is not a definitive match" % (geo_file, filename))
    #     except ValueError as oops:
    #         LOG.warning("problem setting /N_GEO_Ref on %s" % filename)
    #         #            geo_problem_runs.add(granule_id)
    #         good = False

    move_products_to_work_directory(granule_output_dir,ADL_ATMS_PRODUCT_IDS)

    stat = ""
    if good is True:
        stat = 'Granule %s Completed without problems. ' % granule_id

    if good is False:
        stat = 'Granule %s Completed with problems. ' % granule_id

    report_gran_completion(good,stat)

    result = {"GOOD": str(good)}

    return result


def run_all_granules(work_dir, granules_to_process,trim=None,  setup_only=False, CLEANUP="True", nprocs="1", **additional_env):
    """run each ATMS SDR XML input in sequence
    return the list of granule IDs which crashed, and list of granule ids which did not create output
    """
    if not trim: trim = [0, 0]

    global total_granules

    crashed_runs = set()
    no_output_runs = set()
    geo_problem_runs = set()
    bad_log_runs = set()
    first = True
    noncritical_problem = 0
    # # obtain pre-existing granule list
    # satpat = os.path.join(work_dir, 'SATMS*.h5')
    # tatpat = os.path.join(work_dir, 'TATMS*.h5')
    # # create inventory of existing products in the work directory; there should be none
    # _, tatms = h5_xdr_inventory(tatpat, TDR_GRANULE_ID_ATTR_PATH)
    # prior_granules, satms = h5_xdr_inventory(satpat, SDR_GRANULE_ID_ATTR_PATH)
    # prior_granules = set(prior_granules.keys())
    # if tatms or satms:
    #     LOG.warning("This working directory already
    # has products in it! Starting with a clean working directory is strongly recommended.")

    #
    # LOG.debug("building XML files for %d granules" % len(granules_to_process))
    # xml_files_to_process = generate_sdr_xml(work_dir, granules_to_process)

    total_granules = len(granules_to_process)
    LOG.info('%d granules to process' % (len(granules_to_process)))

    granules_to_process = trim_the_granules(granules_to_process,ATMS_GRAN_ID_DELTA, trim=trim)

    fnxml = 'sdr_atms_%s.xml'

    argument_dictionaries,  granules_to_process = setup_multi_process(work_dir,
                                                                         granules_to_process,
                                                                         fnxml,
                                                                         XML_TMPL_ATMS_SDR)


    total_granules = len(granules_to_process)
    start_out_good(total_granules)

    keep_all_output = False
    results = []
    #results = map (run_and_check_single_viirs_sdr, argument_dictionaries )
    if len(argument_dictionaries) > 0:

        nprocs=check_number_processors(int(nprocs))
        gig_recomended = GIG_MEMORY_PER_PROC
        check_memory(nprocs, gig_recomended)

        parallel = Pool(int(nprocs))

        try:
            results = parallel.map_async(run_and_check_single_granule, argument_dictionaries).get(9999999)
        except KeyboardInterrupt:
            LOG.error("Got exception, stopping workers and exiting.")
            parallel.terminate()
            parallel.join()
            sys.exit(1)

    for result in results[1:-1]:
    # do not fail on first or last granule - these can have errors
        if result["GOOD"] != "True":
            LOG.debug("result check %s" % result["GOOD"])
            keep_all_output = True
            noncritical_problem += 1

    if CLEANUP == str(True) and keep_all_output is False:
        for args in argument_dictionaries:
            granule_dir = args['GRANULE_OUTPUT_DIR']
            LOG.debug("Remove: " + granule_dir)
            if os.path.exists(granule_dir):
                shutil.rmtree(granule_dir)

                # clean up the science data only if you there are no other granule directories left

        remaining_files_and_directories = glob.glob(os.path.join(work_dir, "NPP*"))

        if len(remaining_files_and_directories) == 0:
            log_dir = os.path.join(work_dir, "log")
            cleanup(work_dir, 'sdr_atms_*.xml', 'ProSdrAtmsController.exe_*',
                    log_dir, additional_env['LINKED_ANCILLARY'])
    return noncritical_problem




def cleanup(work_dir, xml_glob, log_dir_glob, *more_dirs):
    "upon successful run, clean out work directory"
    LOG.info("cleaning up work directory...")
    for fn in glob.glob(os.path.join(work_dir, '????????-?????-????????-????????.*')):
        LOG.debug('removing %s' % fn)
        os.unlink(fn)
    for fn in glob.glob(os.path.join(work_dir, xml_glob)):
        LOG.debug('removing task file %s' % fn)
        os.unlink(fn)
    for dirname in glob.glob(os.path.join(work_dir, log_dir_glob)):
        LOG.debug('removing logs in %s' % dirname)
        rmtree(dirname, ignore_errors=True)
    for dirname in more_dirs:
        LOG.debug('removing %s' % dirname)
        rmtree(dirname, ignore_errors=True)


def populate_ancillary_links(anc_dir, work_dir, granules_to_process, allow_cache_update=True):
    """
    search for dynamic ancillary data (polar wander and TLE) for the granules we intend to process
    if it's not in the cache already, download it
    softlink those into the workspace
    FUTURE: we currently doesn't operate correctly without this, since we rely on the cache scripts
      to tell us where to find the ancillary data whether it was downloaded or not. cache updating
      should be a flag to adl_anc_retrieval routines which permits/denies remote data fetching
    """
    LOG.info("downloading TLE and PolarWander ancillary into cache and linking into workspace")
    polar_files = adl_anc_retrieval.service_remote_ancillary(work_dir, granules_to_process, adl_anc_retrieval.kPOLAR,
                                                             allow_cache_update=allow_cache_update)
    tle_files = adl_anc_retrieval.service_remote_ancillary(work_dir, granules_to_process, adl_anc_retrieval.kTLE,
                                                           allow_cache_update=allow_cache_update)
    all_dyn_anc = list(polar_files) + list(tle_files)
    LOG.debug('dynamic ancillary files: %s' % repr(all_dyn_anc))
    LOG.info("Link the required ancillary data into the workspace")
    link_ancillary_to_work_dir(anc_dir, all_dyn_anc)

    #    LOG.info("linking static and override ancillary data into workspace")
    search_dirs = ([CSPP_RT_ANC_CACHE_DIR] if CSPP_RT_ANC_CACHE_DIR is not None else []) + [ADL_ANC_DIRS]
    link_ancillary_to_work_dir(anc_dir, anc_files_needed(ADL_ATMS_ANC_GLOBS, search_dirs))
    adl_anc_retrieval.update_orbit_number_in_asc_file(work_dir,anc_dir)


def check_ancillary_effectivity(granules_to_process, anc_dir):
    "verify that we have enough polarwander coverage on all granules, returning filtered list of granules to process"
    LOG.debug('collecting information about available polarwander records')
    wanders = list(skim_dir(anc_dir, required_keys=POLARWANDER_REQUIRED_KEYS,
                            N_Collection_Short_Name='USNO-PolarWander-UT1-ANC-Int'))
    if len(wanders) == 0:
        LOG.error('No polar-wander files were found!')
    n_granules = len(granules_to_process)
    granules_to_process = [gran for gran in granules_to_process if effective_anc_contains(wanders, gran)]
    if n_granules > len(granules_to_process):
        LOG.error(
            'Insufficient polar wander coverage for %d of the candidate granules. '
            'Recommend fetching new PolarWander files.' % (
                n_granules - len(granules_to_process)))
    return granules_to_process


def atms_sdr(work_dir, h5_names, trim=None, setup_only=False, allow_cache_update=True,
             debug=False, compress=True, aggregate=True, nprocs="1"):
    "process ATMS RDR data in HDF5 format (any aggregation) to SDRs in an arbitrary work directory"

    if not trim: trim = [0, 0]

    log_dir, anc_dir = build_workspace(work_dir)

    #   unpacking_problems = unpack_h5s(work_dir, h5_names)

    unpacking_problems = rdr_unpack.unpack_rdr(work_dir, h5_names, allow_cache_update=allow_cache_update)

    # read through ascii metadata and build up information table
    LOG.info('sifting through metadata to find ATMS SDR processing candidates')
    granules_to_process = list(sift_metadata_for_atms_sdr(work_dir))
    LOG.debug(', '.join(x['N_Granule_ID'] for x in granules_to_process))
    if not granules_to_process:
        LOG.warning("Found no granules to process!")
        return get_return_code(unpacking_problems, num_xml_files_to_process=0, num_no_output_runs=0,
                               noncritical_problem=False, environment_error=0)

    xml_files_to_process = []
    try:

        populate_ancillary_links(anc_dir, work_dir, granules_to_process, allow_cache_update=allow_cache_update)

        granules_to_process = check_ancillary_effectivity(granules_to_process, anc_dir)

        # build XML configuration files for jobs that can be run

        LOG.info("running SDR on XML files")
        cleanup = str(not debug)
        noncritical_problem=run_all_granules(work_dir, granules_to_process, trim=trim,setup_only=setup_only, CLEANUP=cleanup, WORK_DIR=work_dir,
                         LINKED_ANCILLARY=anc_dir, ADL_HOME=ADL_HOME, CSPP_RT_ANC_TILE_PATH=CSPP_RT_ANC_TILE_PATH,
                         nprocs=nprocs)

        # considered a noncritical problem if there were any crashed runs, runs that produced no output,
        # runs where Geo failed, or runs where ADL logs indicated a problem
        #        noncritical_problem = crashed_runs or no_output_runs or geo_problem_runs or bad_log_runs

        if aggregate:
            aggregate_products_par(work_dir, ATMS_SDR_PRODUCTS_AGG,nprocs=nprocs)
            add_geo_attribute_to_aggregates(work_dir, ATMS_SDR_PRODUCTS_AGG_GEO)

        if compress:
            try:
                repack_products(work_dir, ATMS_SDR_PRODUCTS_AGG)
            except KeyError:
                LOG.info(repr(list(SHORTNAME_2_PRODUCTID.keys())))
                raise

        # print final disposition message and get return code
        total_granules=len(xml_files_to_process)
        gran_failure = total_granules - number_good_granules()
        rc = get_return_code(unpacking_problems, total_granules, gran_failure, noncritical_problem, environment_error=False)

    except EnvironmentError:
        LOG.error(traceback.format_exc())

        # print final disposition message and get return code
        rc = get_return_code(unpacking_problems, len(xml_files_to_process), num_no_output_runs=0,
                             noncritical_problem=False, environment_error=True)

    # if no errors or only non-critical errors: clean up
    # if rc == 0 and not debug:
    #     cleanup_workspace(work_dir, 'sdr_atms_*.xml', 'ProSdrAtmsController.exe_*', log_dir, anc_dir)

    return rc


def _test_anc_names():
    "list ancillary files that would be processed"
    print "ancillary files:"
    pprint(list(anc_files_needed(ADL_ATMS_ANC_GLOBS, ADL_ANC_DIRS, [])))


def _test_sdr_granules(work_dir):
    "list granules we'd generate XML for"
    granules_to_process = list(sift_metadata_for_atms_sdr(work_dir))
    from pprint import pprint

    pprint([x['N_Granule_ID'] for x in granules_to_process])
    return granules_to_process


def main():
    import argparse

    desc = """Build ATMS SDR ProSdrAtmsController work directory and run ATMS SDR."""
    parser = argparse.ArgumentParser(description=desc)
    parser.add_argument('-T', '--test',
                        action="store_true", default=False, help="run self-tests")
    parser.add_argument('-l', '--local',
                        action="store_true", default=False, help="disable download of remote ancillary data to cache")
    parser.add_argument('-W', '--work-dir', metavar='work_dir', default='.',
                        help='work directory which all activity will occur in, defaults to current dir')
    parser.add_argument('-t', '--trim',  nargs=2, type=int, metavar='N',
                        default=None,
                        help='Number Trim granules. -t \"Trim Begin\" \"Trim End\" -t 1 1 Trims first and last granule.')


    parser.add_argument('-z', '--zip',
                        action="store_true", default=False, help="compress products with h5repack zip compression")
    parser.add_argument('-a', '--aggregate',
                        action="store_true", default=False, help="aggregate products with nagg")
    parser.add_argument('-d', '--debug', action="store_true", default=False,
                        help='enable debug mode on ADL and avoid cleaning workspace')
    parser.add_argument('-v', '--verbosity', action="count", default=0,
                        help='each occurrence increases verbosity 1 level through ERROR-WARNING-INFO-DEBUG')
    parser.add_argument('filenames', metavar='filename', type=str, nargs='+',
                        help='HDF5 ATMS RDR file/s to process')

    parser.add_argument('-p', '--processor',
                        type=int, default=1, help="Number of processors to use for granule processing")

    args = parser.parse_args()

    levels = [logging.ERROR, logging.WARN, logging.INFO, logging.DEBUG]
    level = levels[args.verbosity if args.verbosity < 4 else 3]
    work_dir = check_and_convert_path("WORK_DIR", args.work_dir)
    d = dt.datetime.now()
    timestamp = d.isoformat()
    logname = "atms_sdr." + timestamp + ".log"
    logfile = os.path.join(work_dir, logname)
    configure_logging(level, FILE=logfile)

    if args.aggregate is True:
        if len(args.filenames) > 1:
            LOG.error("Only one input allowed with aggregate option")
            sys.exit(2)

    dpe_version = check_existing_env_var('DPE_VER', default_value=None)
    LOG.info("product version: DPE_VER for ATMS SDR  %s" % dpe_version)


    work_dir = os.path.abspath(args.work_dir)

    LOG.info('CSPP execution work directory is %r' % work_dir)


    if args.test:
        check_env(work_dir)
        _test_anc_names()
        grans = _test_sdr_granules(work_dir)
        if grans:
            LOG.info('building XML files')
            generate_sdr_xml(work_dir, grans)
        sys.exit(2)
    if not args.trim:args.trim = [0, 0]

    if not args:
        parser.print_help()
        return 9
    check_anc_cache(check_write=not args.local)

    check_env(work_dir)

    rc = 1
    try:

        rc = atms_sdr(work_dir, args.filenames, allow_cache_update=not args.local,
                      debug=args.debug,trim=args.trim,  aggregate=args.aggregate, compress=args.zip, nprocs=str(args.processor))

        if rc == 0 and not args.debug:
            os.remove(logfile)

    except Exception:
        LOG.error(traceback.format_exc())

    return rc


if __name__ == '__main__':
    sys.exit(main())
