# !/usr/bin/env python
# encoding: utf-8
"""
$Id: adl_anc_retrieval.py 2496 2015-12-23 17:53:05Z scottm $

Purpose: Provide interface to bash ancillary retrieval scripts.

Input:

Output:


Details:

   Main entry point-   service_remote_ancillary(work_dir,granules_to_process, ancillary_type )
   Given the directory containg the spacecraft data, the science granules and type of ancillary data.
   Return a list of the required ancillary files.

Preconditions:



Copyright (c) 2011 University of Wisconsin Regents.
Licensed under GNU GPLv3.
"""
import os, sys, logging, glob
from datetime import datetime
from adl_common import CSPP_RT_HOME
from ancillary_control import retreive_rsbautocal, retreive_polar, retreive_tle
from adl_common import AscLineParser
import npp_orb_num
import fileinput
from adl_common import _replaceAll
# every module should have a LOG object
LOG = logging.getLogger(__name__)

# add ancillary scripts by
# define constant identifier
# define command string
# add identifier and command to dictionary

kPOLAR = "POLAR"
kTLE = "TLE"
kGRIB = "GRIB"
kNISE = "NISE"
kRSB = 'RSB'

# default=os.path.join(CSPP_RT_HOME,"common","ADL")
# ADL_HOME = check_and_convert_env_var('ADL_HOME',default_value=default)


REMOTE_POLAR_WANDER = os.path.join(CSPP_RT_HOME, 'common', 'get_anc_jpss_polarwander.bash')
REMOTE_TLE = os.path.join(CSPP_RT_HOME, 'common', 'get_anc_jpss_tle.bash')
REMOTE_GRIB = os.path.join(CSPP_RT_HOME, 'common', 'test_run_jpss_product.csh')
REMOTE_NISE = os.path.join(CSPP_RT_HOME, 'common', 'get_anc_jpss_nise.csh')

ancillary_cmds = {kPOLAR: REMOTE_POLAR_WANDER,
                  kTLE: REMOTE_TLE,
                  kGRIB: REMOTE_GRIB,
                  kNISE: REMOTE_NISE,
                  kRSB: None,}

# format strings for time value conversions to shell script parameters
ancillary_strftime = {kPOLAR: "%Y%j %H%M",
                      kTLE: "%Y%j",
                      kGRIB: "%Y%m%d",
                      kNISE: "%Y%j",
                      kRSB: "%Y%j %H:%M:%S",
                      }

class NppFileNameParser(object):
    def time_range(self, file_name):
        day, begin_time, end_time = self.extract_time_range_tokens(file_name)
        return [self.time_from_tokens(day, t) for t in (begin_time, end_time)]

    def extract_time_range_tokens(self, file_name):
        return file_name.split('_')[2:5]

    def time_from_tokens(self, day, time):
        return datetime.strptime(day[1:] + time[1:], '%Y%m%d%H%M%S%f')

def update_orbit_number_in_asc_file(work_dir, anc_dir):
    LOG.debug("update_orbit_number_in_asc_file %s"%work_dir)
    asc_files = glob.glob(os.path.join(work_dir, "*.asc"))
    oldline = "(\"BeginningOrbitNumber\" UINTEGER EQ 1)"
    orb_num = 0
    range_date_time = "RangeDateTime"
    npp_orb = npp_orb_num.NppOrbNum()
    for asc_file in asc_files:
        for line in fileinput.input(asc_file):
            if range_date_time in line:
                # get time of data

                dt = AscLineParser().time_range(line)
                # tle_file=get_tle( dt , allow_cache_update=False )
                tle_files = glob.glob(os.path.join(anc_dir,'*.TLE-AUX*'))

                # get the tle file
                # get the orbit number
                for tle in tle_files:
                    this_orb = npp_orb.usedt(dt, tle)
                    if this_orb != orb_num and this_orb > 2 :
                            orb_num = this_orb
                            LOG.debug(line)
                            LOG.debug("Orbit %s" % orb_num)

                newline = "(\"BeginningOrbitNumber\" UINTEGER EQ %d)" % (int(orb_num))

        fileinput.close()
        _replaceAll(asc_file, oldline, newline)



def service_remote_ancillary(work_dir, granules_to_process, ancillary_type, allow_cache_update=True):
    """
    Check the DB ancillary site and update ancillary files if needed.  Return a list of
    file names to use with these granules.

    work_dir - directory contain input data.
    granules_to_process - List of granules names
    ancillary type - String defining type of ancillary data
    """
    anc_cmd = ancillary_cmds[ancillary_type]
    time_fmt = ancillary_strftime[ancillary_type]

    ancillary_file_set = set()
    date_set = set()

    for gran_name in granules_to_process:
        startTimeDateObj = gran_name['StartTime']
        date = startTimeDateObj.strftime(time_fmt)
        date_set.add(date)

    LOG.debug("dates of interest: %r" % date_set)


    LYJ="0"

    for date in sorted(date_set):

        YJ=date.split(' ')[0]

        if YJ != LYJ :
            LOG.info( 'recheck %s %s'%( YJ,LYJ) )
            LYJ = YJ


            desired_time = datetime.strptime(date, time_fmt)
            if ancillary_type == kRSB:
                needed_ancillary_files = retreive_rsbautocal(work_dir, desired_time,allow_remote=allow_cache_update)

            if ancillary_type == kPOLAR:
                needed_ancillary_files = retreive_polar(work_dir, desired_time,allow_remote=allow_cache_update)

            if ancillary_type == kTLE:
                needed_ancillary_files = retreive_tle(work_dir, desired_time,allow_remote=allow_cache_update)

            LOG.info('files back %s %s' % (str(ancillary_type), str(needed_ancillary_files)))

            for pathname in needed_ancillary_files:
                ancillary_file_set.add(pathname)
        else :
            LOG.debug('already checked  %s'%YJ)

    LOG.debug("ancillary file set: %r" % ancillary_file_set)

    return list(sorted(ancillary_file_set))


def test():
    rdr_name = "RNSCA-RVIRS_npp_d20130124_t2021023_e2029472_b00001_c20130124203202735000_nfts_drl.h5"
    tmp_function(rdr_name)


def main():
    rdr_name = "RNSCA-RVIRS_npp_d20130124_t2021023_e2029472_b00001_c20130124203202735000_nfts_drl.h5"
    tmp_function(rdr_name)
    return 0


if __name__ == '__main__':
    sys.exit(main())
